using UnityEngine;
using System.Collections.Generic;
using System;

public class FOVController : MonoBehaviour {
	
	public const float fovmin = 40.0f;
	public const float fovmax = 120.0f;
	
	public const int zoommin = -5;
	public const int zoommax = 5;
	
	public int zoomamount = 0;
	
	public GameObject meshcontainer;
	public GameObject lightcontainer;
	
	// Use this for initialization
	void Start ()
	{
		FixUVS();
	}
	
	public void OnJump ()
	{
		zoomamount = zoommin;
		Camera.main.GetComponent<AudioHighPassFilter>().cutoffFrequency = 1000;
		//Camera.main.GetComponent<MotionBlur>().blurAmount = 0.8f;
		UpdateFOV();
	}
	
	public void OnLand ()
	{
		zoomamount = 0;
		//Camera.main.GetComponent<MotionBlur>().blurAmount = 0.0f;
		Camera.main.GetComponent<AudioHighPassFilter>().cutoffFrequency = 10;
		UpdateFOV();
	}
	
	// Update is called once per frame
	void Update ()
	{
		if (Application.platform != RuntimePlatform.WindowsWebPlayer && Application.platform != RuntimePlatform.OSXWebPlayer)
		{
			if (Input.GetKeyDown(KeyCode.Escape))
			{
				Application.Quit();
			}
		}
		
		if (Input.GetKeyDown(KeyCode.R))
		{
			Application.LoadLevel(0);
		}
		
		if (Input.GetButtonDown("Zoom In"))
		{
		//	zoomamount = Mathf.Max(zoomamount - 1, zoommin);
		//	UpdateFOV();
		}
		if (Input.GetButtonDown("Zoom Out"))
		{
		//	zoomamount = Mathf.Min(zoomamount + 1, zoommax);
		//	UpdateFOV();
		}	
	}
	
	private void LineStuff (Delegate f)
	{
		/*
		charContr = GetComponent<CharacterController>();
		Vector3 p1 = transform.position + charContr.center + Vector3.up * (-charContr.height * 0.5f + charContr.radius);
		Vector3 p2 = p1 + Vector3.up * (charContr.height - 2 * charContr.radius);
		
		//top + bottom caps
		for (int i = 0; i < 15; i++)
		{
			Vector3 runit = UnityEngine.Random.onUnitSphere;
			if (runit.y > 0)
				runit = new Vector3(runit.x, -runit.y, runit.z);
			Ray ray1 = new Ray(p1, runit);
			f.DynamicInvoke(ray1);
			
			Ray ray2 = new Ray(p2, new Vector3(runit.x, -runit.y, runit.z));
			f.DynamicInvoke(ray2);			
		}
		
		//points along cylinder
		for (int i = 0; i < 5; i++)
		{
			Vector3 hvector = p1 + (i * (p2 - p1)) / 5;
			
			for (int j = 0; j < 10; j++)
			{
				Vector2 runit = UnityEngine.Random.insideUnitCircle;
				Ray ray = new Ray(hvector, new Vector3(runit.x, 0, runit.y));
				f.DynamicInvoke(ray);			
			}
		}
		*/
	}
	
	void OnDrawGizmos ()
	{
		LineStuff(new Action<Ray>(DrawRay));
	}
	
	private void FixUVS ()
	{
		for (int i = 0; i < meshcontainer.transform.GetChildCount(); i++)
		{
			GameObject child = meshcontainer.transform.GetChild(i).gameObject;
			MeshFilter mf = child.GetComponent<MeshFilter>();
			MeshCollider mc = child.GetComponent<MeshCollider>();
			mc.convex = false;
			
			Mesh mesh = mf.mesh;
			
			Vector3[] vertices = mesh.vertices;
			Vector3[] normals = mesh.normals;
			Vector2[] uvs = mesh.uv;
			
			for (int j = 0; j < uvs.Length; j++)
			{
				//Vector2 uv = uvs[i];
				Vector3 v = child.transform.TransformPoint(vertices[j]);
				Vector3 normal = child.transform.TransformDirection(normals[j]);
				Quaternion rot = Quaternion.FromToRotation(normal, new Vector3(0, 1, 0));
				Vector3 transformed = rot * v;
				uvs[j] = new Vector2(transformed.x/10, transformed.z/10);
			}
			
			mesh.uv = uvs;
			mesh.RecalculateBounds();
			mesh.Optimize();
		}
	}
	
	private void UpdateFOV ()
	{
		float targetfov = fovmin + ((fovmax-fovmin)*(zoomamount-zoommin))/(zoommax-zoommin);
		Matrix4x4 c2w = Camera.mainCamera.cameraToWorldMatrix;
		Matrix4x4 w2c = Camera.mainCamera.worldToCameraMatrix;
		
		Matrix4x4 c2s0 = Camera.mainCamera.projectionMatrix;
		
		Camera.mainCamera.fov = targetfov;
		
		Matrix4x4 s2c1 = Camera.mainCamera.projectionMatrix.inverse;
		
		for (int i=0;i< meshcontainer.transform.GetChildCount();i++)
		{
			GameObject child = meshcontainer.transform.GetChild(i).gameObject;
			MeshFilter mf = child.GetComponent<MeshFilter>();
			MeshCollider mc = child.GetComponent<MeshCollider>();
			
			Mesh mesh = mf.mesh;
			
			Vector3[] vertices = mesh.vertices;
			
			for (int j=0;j<vertices.Length;j++)
			{
				Vector3 pre_local = vertices[j];
				Vector3 pre_world = child.transform.TransformPoint(pre_local);
				
				Vector3 pre_camera = w2c.MultiplyPoint(pre_world);
				
				Vector3 screen = c2s0.MultiplyPoint(pre_camera);
				
				Vector3 post_camera = s2c1.MultiplyPoint(screen);
				
				Vector3 post_world = c2w.MultiplyPoint(post_camera);
				
				Vector3 post_local = child.transform.InverseTransformPoint(post_world);
				
				vertices[j]= post_local;
			}
			
			mesh.vertices = vertices;
			
			mc.sharedMesh=null;
			mc.sharedMesh = mesh;
			mc.sharedMesh.RecalculateBounds();
			//mf.mesh.RecalculateNormals();
			mc.sharedMesh.Optimize();
		}
		
		for (int i = 0; i < lightcontainer.transform.GetChildCount(); i++)
		{
			GameObject child = lightcontainer.transform.GetChild(i).gameObject;
			
			Vector3 pre_world = child.transform.position;
					
			Vector3 pre_camera = w2c.MultiplyPoint(pre_world);
			
			Vector3 screen = c2s0.MultiplyPoint(pre_camera);
			
			Vector3 post_camera = s2c1.MultiplyPoint(screen);
			
			child.transform.position = c2w.MultiplyPoint(post_camera);							
		}
				
		LineStuff(new Action<Ray>(RayCheck));		
	}
	
	private CharacterController charContr ;
	
	private void DrawRay (Ray ray)
	{
		float radius = charContr.radius;
		Gizmos.DrawLine(ray.origin, ray.origin + radius * ray.direction);
	}
	
	private void RayCheck (Ray ray)
	{
		RaycastHit hit;
		float radius = charContr.radius;
		Ray ray1 = new Ray(ray.origin - ray.direction*radius, ray.direction);
		if (Physics.Raycast(ray1, out hit, 2 * radius, 1 << 8))
		{
			Debug.Log("OFFSETTING" + hit.distance);
			Vector3 v = hit.point - ray.origin;
			float l = v.magnitude;
			
			transform.position = transform.position + (radius*1.1f - l) * v.normalized;;
		}
	}
}
